@@ -4,8 +4,8 @@
  *
  * Code Contributions By:	Bram Moolenaar			mool@oce.nl
  *							Tim Thompson			twitch!tjt
- *							Tony Andrews			onecom!wldrdg!tony 
- *							G. R. (Fred) Walter		watmath!watcgl!grwalter 
+ *							Tony Andrews			onecom!wldrdg!tony
+ *							G. R. (Fred) Walter     watmath!watcgl!grwalter
  */
 /*
  * search.c: code for normal mode searching commands
@@ -16,269 +16,190 @@
 /* modified Henry Spencer's regular expression routines */
 #include "regexp.h"
 
-#ifdef	MEGAMAX
-overlay "search"
-#endif
-
-static void	mapstring __ARGS((char *, char *));
 static int inmacro __ARGS((int, char *));
-static int cls();
+static int cls __ARGS((void));
 
 char		emsg_inval[] = "Invalid search string";
+char		emsg_interrupted[] = "Interrupted";
 
 /*
  * This file contains various searching-related routines. These fall into
- * three groups: string searches (for /, ?, n, and N), character searches
- * within a single line (for f, F, t, T, etc), and "other" kinds of searches
- * like the '%' command, and 'word' searches.
+ * three groups:
+ * 1. string searches (for /, ?, n, and N)
+ * 2. character searches within a single line (for f, F, t, T, etc)
+ * 3. "other" kinds of searches like the '%' command, and 'word' searches.
  */
 
 /*
  * String searches
  *
- * The string search functions are divided into three levels:
- * Lowest:  strsearch(); does not take a count, called by seachit().
- * Middle:  searchit(); called by dosearch() and docmdline().
+ * The string search functions are divided into two levels:
+ * lowest:	searchit(); called by dosearch() and docmdline().
  * Highest: dosearch(); changes Curpos, called by normal().
  *
- * The actual string matching is done using Henry Spencer's regular expression
- * library. Therefore some characters have to be mapped from the vi syntax to
- * the regexp syntax.
- */
-
-bool_t begword; 			/* does the search include a 'begin word' match */
-
-/*
- * mapstring(s) - map magic string characters, depending on magic option
+ * The last search pattern is remembered for repeating the same search.
+ * This pattern is shared between the :g, :s, ? and / commands.
+ * This is in myregcomp().
+ *
+ * The actual string matching is done using a heavily modified version of
+ * Henry Spencer's regular expression library.
  *
- * input  magic 			  nomagic
- *	(		\(					\(
- *  )		\)					\)
- *	|		\|					\|
- *  ?	    \?					\?
- *  +       \+					\+
- *	[		[					\[
- *	]		]					\]
- *	*		*					\*
- *	.		.					\.
- *	\<		([^a-zA-Z0-9_]|^)	([^a-zA-Z0-9_]|^)
- *	\>		([^a-zA-Z0-9_]|$)	([^a-zA-Z0-9_]|$)
- *	\(		(					(
- *	\)		)					)
- *	\/		/					/
- *	\|		|					|
- *  \?		?					?
- *  \+		+					+
- *	\[		\[					[
- *	\]		\]					]
- *	\*		\*					*
- *	\.		\.					.
- *  \x		\x					\x		(if the above don't match)
- *  x		x					x		(if the above don't match)
  */
-void
-mapstring(s, ns)
-	register char  *s;
-	register char  *ns;
+
+static char 	*search_pattern = NULL;  /* previous search pattern */
+static bool_t	want_start; 			 /* looking for start of line? */
+
+/*
+ * translate search pattern for regcomp()
+ */
+	regexp *
+myregcomp(pat)
+	char *pat;
 {
-    register char  *p;
+	regexp *retval;
 
-    if (strncmp(s, "\\<", 2) == 0)	/* begin of word handled differently */
-	begword = TRUE;
-    else
-	begword = FALSE;
-
-    for (p = ns; *s; s++)
-    {
-		if (*s == '\\') 	/* backslash sequence */
+	if (pat == NULL || *pat == NUL)     /* use previous search pattern */
+	{
+		if (search_pattern == NULL)
 		{
-			++s;
-			if (*s == '<' || *s == '>')		/* word boundary */
-			{
-				strcpy(ns, "([^a-zA-Z0-9_]|");
-				strcat(ns, *s == '<' ? "^)" : "$)");
-				ns += strlen(ns);
-			}
-			else
-			{
-				if (P(P_MAGIC) == TRUE && index("*.[]", *s) != NULL ||
-							index("/|()?+*.[]", *s) == NULL)		/* copy the backslash*/
-					*ns++ = '\\';
-				*ns++ = *s;
-			}
-		}
-		else				/* not a backslash sequence */
-		{
-			if (index("()|?+", *s) != NULL ||
-						P(P_MAGIC) == FALSE && index(".*[]", *s) != NULL)
-				*ns++ = '\\';
-			*ns++ = *s;
+			emsg("No previous search pattern");
+			return (regexp *) NULL;
 		}
+		pat = search_pattern;
 	}
-	*ns = NUL;
+	else
+	{
+		if (search_pattern != NULL)
+			free(search_pattern);
+		search_pattern = strsave(pat);
+		reg_magic = P(P_MAGIC);		/* Magic sticks with the r.e. */
+	}
+	want_start = (*pat == '^');		/* looking for start of line? */
+	reg_ic = P(P_IC);				/* tell the regexec routine how to search */
+	retval = regcomp(pat);
+	return retval;
 }
 
 /*
- * Lowest level string search function.
- * search for string 'str' in direction 'dir'
- * if 'end' is TRUE, position cursor on last char of match
- */
-static bool_t
-strsearch(dir, str, pos, end)
-	int			dir;
-	char		*str;
-	FPOS		*pos;
-	bool_t		end;
-{
-	linenr_t		lnum;
-	linenr_t		startlnum = pos->lnum;
-	regexp		   *prog;
-	register char  *s;
-	char		   *ptr;
-	register int	i;
-	bool_t			want_start = (*str == '^'); /* looking for start of line? */
-	register char  *match, *matchend;
-	int				loop;
-
-	/* make sure str isn't empty */
-	if (str == NULL || *str == NUL)
-		return NULL;
-
-	if ((prog = regcomp(str)) == NULL)
-	{
-		emsg(emsg_inval);
-		return NULL;
-	}
-
-	i = pos->col + dir;		/* search starts one postition away */
-	lnum = pos->lnum;
-
-	if (dir == BACKWARD)
-	{
-		if (begword)
-			--i;	/* so we don't get stuck on one match */
-		if (i < 0)
-			--lnum;
-	}
-
-	for (loop = 0; loop != 2; ++loop)	/* do this twice if 'wrapscan' is set */
-	{
-		for ( ; lnum > 0 && lnum <= line_count; lnum += dir, i = -1)
-		{
-			s = ptr = nr2ptr(lnum);
-			if (dir == FORWARD && i > 0)	/* first line for forward search */
-			{
-				if (want_start || strlen(s) <= i) 	/* match not possible */
-					continue;
-				s += i;
-			}
-
-			if (regexec(prog, s, dir == BACKWARD || i <= 0))
-			{							/* match somewhere on line */
-				match = prog->startp[0];
-				matchend = prog->endp[0];
-				if (dir == BACKWARD && !want_start)
-				{
-					/*
-					 * Now, if there are multiple matches on this line, we have to
-					 * get the last one. Or the last one before the cursor, if we're
-					 * on that line.
-					 */
-					while (regexec(prog, prog->startp[0] + 1, FALSE))
-					{
-						if ((i >= 0) && ((prog->startp[0] - s) > i))
-							break;
-						match = prog->startp[0];
-						matchend = prog->endp[0];
-					}
-
-					if ((i >= 0) && ((match - s) > i))
-						continue;
-				}
-
-				pos->lnum = lnum;
-				if (end)
-					pos->col = (int) (matchend - ptr - 1);
-				else
-					pos->col = (int) (match - ptr);
-				free((char *) prog);
-				return (TRUE);
-			}
-			breakcheck();		/* stop if ctrl-C typed */
-			if (got_int)
-				goto fail;
-
-			if (loop && lnum == startlnum)	/* if second loop stop where started */
-				break;
-		}
-
-		if (!P(P_WS))	/* If wrapscan isn't set, bag the search now */
-			goto fail;
-
-		if (dir == BACKWARD)	/* start second loop at the other end */
-			lnum = line_count;
-		else
-			lnum = 1;
-	}
-
-fail:
-	free((char *) prog);
-	return FALSE;
-}
-
-/*
- * Middle level search function.
+ * lowest level search function.
  * Search for 'count'th occurrence of 'str' in direction 'dir'.
  * Start at position 'pos' and return the found position in 'pos'.
  * Return 1 for success, 0 for failure.
  */
 searchit(pos, dir, str, count, end)
 	FPOS	*pos;
-	int		dir;
+	int 	dir;
 	char	*str;
-	int		count;
+	long	count;
 	bool_t	end;
 {
-	char		   *buf;		/* buffer for translated string */
-	int				c;
-/*
- * allocate a buffer to translate into;
- * Maximum expansion is replacing '\<' by 17 chars, so making it 9 times larger
- * will be more than enough.
- */
-	if ((buf = alloc(strlen(str) * 9 + 1)) == NULL)
-		buf = str;
-	else
-		mapstring(str, buf);
+	int 			found;
+	linenr_t		lnum;
+	linenr_t		startlnum;
+	regexp		   *prog;
+	register char  *s;
+	char		   *ptr;
+	register int	i;
+	register char  *match, *matchend;
+	int 			loop;
 
+	if ((prog = myregcomp(str)) == NULL)
+	{
+		emsg(emsg_inval);
+		return 0;
+	}
 /*
  * find the string
  */
-	while (count-- && (c = strsearch(dir, buf, pos, end)))
-		;
+	found = 1;
+	while (count-- && found)    /* stop after count matches, or no more matches */
+	{
+		startlnum = pos->lnum;	/* remember start of search for detecting no match */
+		found = 0;				/* default: not found */
 
-	if (buf != str)
-		free(buf);
+		i = pos->col + dir; 	/* search starts one postition away */
+		lnum = pos->lnum;
 
-	if (!c) 			/* did not find it */
+		if (dir == BACKWARD)
+		{
+			if (i < 0)
+				--lnum;
+		}
+
+		for (loop = 0; loop != 2; ++loop)   /* do this twice if 'wrapscan' is set */
+		{
+			for ( ; lnum > 0 && lnum <= line_count; lnum += dir, i = -1)
+			{
+				s = ptr = nr2ptr(lnum);
+				if (dir == FORWARD && i > 0)    /* first line for forward search */
+				{
+					if (want_start || strlen(s) <= i)   /* match not possible */
+						continue;
+					s += i;
+				}
+
+				if (regexec(prog, s, dir == BACKWARD || i <= 0))
+				{							/* match somewhere on line */
+					match = prog->startp[0];
+					matchend = prog->endp[0];
+					if (dir == BACKWARD && !want_start)
+					{
+						/*
+						 * Now, if there are multiple matches on this line, we have to
+						 * get the last one. Or the last one before the cursor, if we're
+						 * on that line.
+						 */
+						while (regexec(prog, prog->startp[0] + 1, (int)FALSE))
+						{
+							if ((i >= 0) && ((prog->startp[0] - s) > i))
+								break;
+							match = prog->startp[0];
+							matchend = prog->endp[0];
+						}
+
+						if ((i >= 0) && ((match - s) > i))
+							continue;
+					}
+
+					pos->lnum = lnum;
+					if (end)
+						pos->col = (int) (matchend - ptr - 1);
+					else
+						pos->col = (int) (match - ptr);
+					found = 1;
+					break;
+				}
+				breakcheck();       /* stop if ctrl-C typed */
+				if (got_int)
+					break;
+
+				if (loop && lnum == startlnum)  /* if second loop stop where started */
+					break;
+			}
+	/* stop the search if wrapscan isn't set, after an interrupt and after a match */
+			if (!P(P_WS) || got_int || found)
+				break;
+
+			if (dir == BACKWARD)    /* start second loop at the other end */
+				lnum = line_count;
+			else
+				lnum = 1;
+		}
+		if (got_int)
+			break;
+	}
+
+	free((char *) prog);
+
+	if (!found)             /* did not find it */
 	{
 		if (got_int)
-				emsg("Interrupted");
+				emsg(emsg_interrupted);
 		else
 				emsg("Pattern not found");
 		return 0;
 	}
 
-	/*
-	 * This is kind of a kludge, but its needed to make 'beginning of word'
-	 * searches land on the right place.
-	 */
-	if (begword)
-	{
-		c = gchar(pos);
-		if (pos->col != 0 || !isidchar(c))
-			pos->col += 1;
-	}
 	return 1;
 }
 
@@ -291,20 +212,18 @@ searchit(pos, dir, str, count, end)
  */
 	bool_t
 dosearch(dir, str, reverse, count)
-	int			dir;
+	int				dir;
 	char		   *str;
 	bool_t			reverse;
-	int				count;
+	long			count;
 {
 	FPOS			pos;		/* position of the last match */
-	register char  *p;
-	register int	c;
-	register int	dirc;
-	static char    *laststr = NULL; /* previous/current search string */
-	static int		lastsdir;		/* previous search direction */
-	static bool_t	lasthasoff;		/* previous/current search has line offset */
-	static int		lastend;		/* previous/current search set cursor at end */
-	static int		lastoff;		/* previous/current line or char offset */
+	int				dirc;
+	static int		lastsdir = FORWARD;	/* previous search direction */
+	static bool_t	lastoffline;/* previous/current search has line offset */
+	static bool_t	lastend;	/* previous/current search set cursor at end */
+	static long 	lastoff;	/* previous/current line or char offset */
+	static bool_t   nosetpm;    /* do not call setpcmark() */
 
 	if (dir == 0)
 		dir = lastsdir;
@@ -314,24 +233,14 @@ dosearch(dir, str, reverse, count)
 		dir = -dir;
 
 	dirc = (dir == FORWARD ? '/' : '?');
-	if (str == NULL || *str == NUL || *str == dirc)
+	if (str == NULL || *str == NUL)     /* use previous string and options */
+		str = "";
+	else
 	{
-		if (laststr == NULL)
-		{
-			beep();
-			return 0;
-		}
-	}
-	else		/* save the string for repeating the search */
-	{
-		if (laststr != NULL)
-			free(laststr);
-		laststr = strsave(str);
-		if (laststr == NULL)
-			return 0;
+		register char  *p;
 
 		/* If there is a matching '/' or '?', toss it */
-		for (p = laststr; *p; ++p)
+		for (p = str; *p; ++p)
 		{
 			if (*p == dirc)
 			{
@@ -342,87 +251,102 @@ dosearch(dir, str, reverse, count)
 				++p;	/* skip next character */
 		}
 
-		lasthasoff = FALSE;
+		lastoffline = FALSE;
 		lastend = FALSE;
+		nosetpm = FALSE;
 		lastoff = 0;
 		switch (*p)
 		{
-			case '+':
-			case '-': 					/* got a line offset */
-						lasthasoff = TRUE;
+			case 'n':   nosetpm = TRUE; /* do not call setpcmark() */
 						break;
-			case 'e':					/* position cursor at end */
+			case '+':
+			case '-':                   /* got a line offset */
+						lastoffline = TRUE;
+						break;
+			case 'e':                   /* position cursor at end */
 						lastend = TRUE;
-			case 's':					/* got a character offset */
+			case 's':                   /* got a character offset */
 						++p;
 		}
-		if (*p == '+' || *p == '-')		/* got a line offset */
-			lastoff = atoi(p);
+		if (*p == '+' || *p == '-')     /* got a line offset */
+			lastoff = atol(p);
 	}
 
-	smsg("%c%s", dirc, laststr);
-	if (lasthasoff || lastend || lastoff)
+	smsg("%c%s", dirc, *str == NUL ? search_pattern : str);
+	if (lastoffline || lastend || lastoff || nosetpm)
 	{
 		outchar(dirc);
-		if (lastend)
+		if (nosetpm)
+			outchar('n');
+		else if (lastend)
 			outchar('e');
-		else if (!lasthasoff)
+		else if (!lastoffline)
 			outchar('s');
 		if (lastoff < 0)
 		{
 			outchar('-');
 			outnum(-lastoff);
 		}
-		else if (lastoff > 0 || lasthasoff)
+		else if (lastoff > 0 || lastoffline)
 		{
 			outchar('+');
 			outnum(lastoff);
 		}
 	}
 
-	gotocmdline(FALSE, NUL);
+	gotocmdline((bool_t)FALSE, NUL);
 	flushbuf();
-	reg_ic = P(P_IC);			/* tell the regexp routines how to search */
 
 	pos = Curpos;
 
-	if (!searchit(&pos, dir, laststr, count, lastend))
+	if (!searchit(&pos, dir, str, count, lastend))
 		return 0;
 
-	if (!lasthasoff)			/* add the character offset to the column */
+	if (!lastoffline)           /* add the character offset to the column */
 	{
-		if (lastoff > 0)		/* offset to the right, check for end of line */
+		if (lastoff > 0)        /* offset to the right, check for end of line */
 		{
+			register long	c;
+			register char  *p;
+
 			p = nr2ptr(pos.lnum) + pos.col + 1;
-			while (lastoff-- && *p++ != NUL)
+			c = lastoff;
+			while (c-- && *p++ != NUL)
 				++pos.col;
 		}
 		else					/* offset to the left, check for start of line */
 		{
+			register long	c;
+
 			if ((c = pos.col + lastoff) < 0)
 				c = 0;
 			pos.col = c;
 		}
 	}
 
-	setpcmark();
+	if (!nosetpm)
+		setpcmark();
 	Curpos = pos;
 	set_want_col = TRUE;
 
-	if (!lasthasoff)
+	if (!lastoffline)
 		return 1;
 
 /*
  * add the offset to the line number.
  */
-	c = Curpos.lnum + lastoff;
-	if (c < 1)
-		Curpos.lnum = 1;
-	else if (c > line_count)
-		Curpos.lnum = line_count;
-	else
-		Curpos.lnum = c;
-	Curpos.col = 0;
+	{
+		register long	c;
+
+		c = Curpos.lnum + lastoff;
+		if (c < 1)
+			Curpos.lnum = 1;
+		else if (c > line_count)
+			Curpos.lnum = line_count;
+		else
+			Curpos.lnum = c;
+		Curpos.col = 0;
+	}
 
 	return 2;
 }
@@ -439,21 +363,21 @@ dosearch(dir, str, reverse, count)
  * position of the character, otherwise move to just before the char.
  * Repeat this 'count' times.
  */
-bool_t
+	bool_t
 searchc(c, dir, type, count)
-	int				c;
+	int 			c;
 	register int	dir;
-	int				type;
-	int				count;
+	int 			type;
+	long			count;
 {
-	static char		lastc = NUL;	/* last character searched for */
+	static char 	lastc = NUL;	/* last character searched for */
 	static int		lastcdir;		/* last direction of character search */
 	static int		lastctype;		/* last type of search ("find" or "to") */
 	register int	col;
 	char			*p;
-	int				len;
+	int 			len;
 
-	if (c != NUL)		/* normal search: remember args for repeat */
+	if (c != NUL)       /* normal search: remember args for repeat */
 	{
 		lastc = c;
 		lastcdir = dir;
@@ -463,7 +387,7 @@ searchc(c, dir, type, count)
 	{
 		if (lastc == NUL)
 			return FALSE;
-		if (dir)		/* repeat in opposite direction */
+		if (dir)        /* repeat in opposite direction */
 			dir = -lastcdir;
 		else
 			dir = lastcdir;
@@ -504,17 +428,17 @@ searchc(c, dir, type, count)
 /*
  * showmatch - move the cursor to the matching paren or brace
  */
-FPOS		   *
+	FPOS		   *
 showmatch()
 {
-	static FPOS	pos;
+	static FPOS pos;
 	char		initc;		/* char under cursor */
 	char		findc;		/* matching char */
 	char		c;
-	int			count = 0;
-	int			i;
-	static char	table[6] = {'(', ')', '[', ']', '{', '}'};
-	int			inquote = 0;
+	int 		count = 0;
+	int 		i;
+	static char table[6] = {'(', ')', '[', ']', '{', '}'};
+	int 		inquote = 0;
 	register char		*p;
 
 	pos = Curpos;
@@ -529,18 +453,18 @@ showmatch()
 	}
 
 foundit:
-	if (i & 1)					/* backward search */
+	if (i & 1)                  /* backward search */
 		findc = table[i - 1];
 	else						/* forward search */
 		findc = table[i + 1];
 
 	for (; !got_int; breakcheck())
 	{
-		if (i & 1)				/* backward search */
+		if (i & 1)              /* backward search */
 		{
-			if (pos.col == 0)	/* at start of line, go to previous one */
+			if (pos.col == 0)   /* at start of line, go to previous one */
 			{
-				if (pos.lnum == 1)		/* start of file */
+				if (pos.lnum == 1)      /* start of file */
 					break;
 				--pos.lnum;
 				p = nr2ptr(pos.lnum);
@@ -551,9 +475,9 @@ foundit:
 		}
 		else					/* forward search */
 		{
-			if (p[pos.col] == NUL)	/* at end of line, go to next one */
+			if (p[pos.col] == NUL)  /* at end of line, go to next one */
 			{
-				if (pos.lnum == line_count)	/* end of file */
+				if (pos.lnum == line_count) /* end of file */
 					break;
 				++pos.lnum;
 				pos.col = 0;
@@ -565,18 +489,18 @@ foundit:
 		c = p[pos.col];
 		if (c == NUL && (pos.col == 0 || p[pos.col - 1] != '\\'))
 			inquote = FALSE;
-		else if (c == '"')					/* count number of quotes */
+		else if (c == '"')                  /* count number of quotes */
 			inquote = !inquote;
-		else if (c == '\'')					/* skip 'x' or '\x' */
+		else if (c == '\'')                 /* skip 'x' or '\x' */
 		{
-			if (i & 1 && pos.col > 1)			/* backward search */
+			if (i & 1 && pos.col > 1)           /* backward search */
 			{
 				if (p[pos.col - 2] == '\'')
 					pos.col -= 2;
 				else if (p[pos.col - 2] == '\\' && pos.col > 2 && p[pos.col - 3] == '\'')
 					pos.col -= 3;
 			}
-			else if (p[pos.col + 1])			/* forward search */
+			else if (p[pos.col + 1])            /* forward search */
 			{
 				if (p[pos.col + 2] == '\'')
 					pos.col += 2;
@@ -584,7 +508,7 @@ foundit:
 					pos.col += 3;
 			}
 		}
-		else if (!inquote)		/* only check for match outside of quotes */
+		else if (!inquote)      /* only check for match outside of quotes */
 		{
 			if (c == initc)
 				count++;
@@ -596,7 +520,7 @@ foundit:
 			}
 		}
 	}
-	return (FPOS *) NULL;		/* never found it */
+	return (FPOS *) NULL;       /* never found it */
 }
 
 /*
@@ -604,11 +528,11 @@ foundit:
  *
  * Return TRUE if a line was found.
  */
-bool_t
+	bool_t
 findfunc(dir, what, count)
-	int		dir;
+	int 		dir;
 	bool_t		what;
-	int		count;
+	long		count;
 {
 	linenr_t	curr;
 
@@ -648,14 +572,13 @@ findfunc(dir, what, count)
  * Return TRUE if the next sentence was found.
  */
 findsent(dir, count)
-		int	dir;
-		int	count;
+		int 	dir;
+		long	count;
 {
 		FPOS			pos, tpos;
 		register int	c;
-		int			r;
-		int			(*func)();
-		int				startlnum;
+		int 			(*func)();
+		int 			startlnum;
 
 		pos = Curpos;
 		if (dir == FORWARD)
@@ -689,14 +612,14 @@ findsent(dir, count)
 
 				/* go back to the previous non-blank char */
 				while ((c = gchar(&pos)) == ' ' || c == '\t' || dir == BACKWARD &&
-										index(".!?)]\"'", c) != NULL)
+										strchr(".!?)]\"'", c) != NULL)
 						if (decl(&pos) == -1)
 								break;
 
 				/* remember the line where the search started */
 				startlnum = pos.lnum;
 
-				for (;;)				/* find end of sentence */
+				for (;;)                /* find end of sentence */
 				{
 						if ((c = gchar(&pos)) == NUL ||
 										pos.col == 0 && startPS(pos.lnum, NUL))
@@ -711,11 +634,11 @@ findsent(dir, count)
 								do
 									if ((c = inc(&tpos)) == -1)
 										break;
-								while (index(")}\"'", c = gchar(&tpos)) != NULL);
+								while (strchr(")}\"'", c = gchar(&tpos)) != NULL);
 								if (c == -1  || c == ' ' || c == '\t' || c == NUL)
 								{
 										pos = tpos;
-										if (gchar(&pos) == NUL)	/* skip NUL at EOL */
+										if (gchar(&pos) == NUL) /* skip NUL at EOL */
 												inc(&pos);
 										break;
 								}
@@ -745,15 +668,13 @@ found:
  * Return TRUE if the next paragraph was found.
  * If 'what' is '{' or '}' we go to the next section.
  */
-bool_t
+	bool_t
 findpar(dir, count, what)
 	register int	dir;
-	int			count;
-	int			what;
+	long			count;
+	int 			what;
 {
 	register linenr_t	curr;
-	register char		*s;
-	register char		*para;
 	bool_t				did_skip;		/* TRUE after separating lines have
 												been skipped */
 	bool_t				first;			/* TRUE on first line */
@@ -820,7 +741,7 @@ inmacro(opt, s)
  */
 startPS(lnum, para)
 	linenr_t	lnum;
-	int			para;
+	int 		para;
 {
 	register char *s;
 
@@ -860,7 +781,7 @@ static int		stype;			/* type of the word motion being performed */
  * 'B', or 'E' motion is being done. In this case, chars. from class 2 are
  * reported as class 1 since only white space boundaries are of interest.
  */
-static int
+	static int
 cls()
 {
 	register int c;
@@ -885,26 +806,26 @@ cls()
  * Returns TRUE if end of the file was reached.
  */
 fwd_word(count, type)
-	int 		count;
-	int			type;
+	long		count;
+	int 		type;
 {
-	int			sclass; 	/* starting class */
+	int 		sclass; 	/* starting class */
 
 	stype = type;
 	while (--count >= 0)
 	{
 		sclass = cls();
-	
+
 		/*
 		 * We always move at least one character.
 		 */
 		if (incCurpos() == -1)
 			return TRUE;
-	
+
 		if (sclass != 0)
 			if (skip_chars(sclass, FORWARD))
 				return TRUE;
-	
+
 		/*
 		 * go to next non-white
 		 */
@@ -915,7 +836,7 @@ fwd_word(count, type)
 			 */
 			if (Curpos.col == 0 && *nr2ptr(Curpos.lnum) == NUL)
 				break;
-	
+
 			if (incCurpos() == -1)
 				return TRUE;
 		}
@@ -929,46 +850,46 @@ fwd_word(count, type)
  * Returns TRUE if top of the file was reached.
  */
 bck_word(count, type)
-	int			count;
-	int			type;
+	long		count;
+	int 		type;
 {
-	int			sclass; 	/* starting class */
+	int 		sclass; 	/* starting class */
 
 	stype = type;
 	while (--count >= 0)
 	{
 		sclass = cls();
-	
+
 		if (decCurpos() == -1)
 			return TRUE;
-	
+
 		if (cls() != sclass || sclass == 0)
 		{
 			/*
 			 * We were at the start of a word. Go back to the end of the prior
 			 * word.
 			 */
-			while (cls() == 0) 	/* skip any white space */
+			while (cls() == 0)  /* skip any white space */
 			{
 				/*
 				 * We'll stop if we land on a blank line
 				 */
 				if (Curpos.col == 0 && *nr2ptr(Curpos.lnum) == NUL)
 					goto finished;
-		
+
 				if (decCurpos() == -1)
 					return TRUE;
 			}
 			sclass = cls();
 		}
-	
+
 		/*
 		 * Move backward to start of this word.
 		 */
 		if (skip_chars(sclass, BACKWARD))
 				return TRUE;
-	
-		incCurpos();					/* overshot - forward one */
+
+		incCurpos();                    /* overshot - forward one */
 finished:
 		;
 	}
@@ -990,11 +911,11 @@ finished:
  * If stop is TRUE and we are already on the end of a word, move one less.
  */
 end_word(count, type, stop)
-	int			count;
-	int			type;
+	long		count;
+	int 		type;
 	bool_t		stop;
 {
-	int			sclass; 	/* starting class */
+	int 		sclass; 	/* starting class */
 
 	stype = type;
 	while (--count >= 0)
@@ -1002,7 +923,7 @@ end_word(count, type, stop)
 		sclass = cls();
 		if (incCurpos() == -1)
 			return TRUE;
-	
+
 		/*
 		 * If we're in the middle of a word, we just have to move to the end of it.
 		 */
@@ -1020,7 +941,7 @@ end_word(count, type, stop)
 			 * We were at the end of a word. Go to the end of the next word.
 			 */
 
-			if (skip_chars(0, FORWARD)) 	/* skip any white space */
+			if (skip_chars(0, FORWARD))     /* skip any white space */
 				return TRUE;
 
 			/*
@@ -1029,16 +950,16 @@ end_word(count, type, stop)
 			if (skip_chars(cls(), FORWARD))
 				return TRUE;
 		}
-		decCurpos();					/* overshot - backward one */
+		decCurpos();                    /* overshot - backward one */
 		stop = FALSE;					/* we move only one word less */
 	}
 	return FALSE;
 }
 
-		int
+	int
 skip_chars(class, dir)
-		int	class;
-		int	dir;
+	int class;
+	int dir;
 {
 		while (cls() == class)
 			if ((dir == FORWARD ? incCurpos() : decCurpos()) == -1)
